'''
This module provides a data analysis window.
'''

#{{{ Import modules
import enthought.etsconfig.api
from enthought.traits.api import HasTraits, Instance, Int, Float, Enum, Bool,\
     Trait, Callable, Button, Array, CInt, CFloat, List, Str
from enthought.traits.ui.api import View, Group, HGroup, VGroup, Item, Readonly, spring,\
     Label, Handler
from enthought.traits.ui.menu import Action, MenuBar, Menu, CloseAction
import enthought.traits.ui.file_dialog as file_dialog
from enthought.enable.api import ColorTrait, ComponentEditor, Component
from enthought.chaco.api import marker_trait, Plot, BarPlot, ArrayPlotData, ArrayDataSource,\
     Legend, OverlayPlotContainer, PlotAxis, DataRange1D, LinearMapper, add_default_axes,\
     add_default_grids, PlotGrid, LinePlot, ScatterPlot
from enthought.chaco.tools.api import LegendTool, PanTool, ZoomTool, RectZoomTool
from enthought.chaco.scales.formatters import NullFormatter
from enthought.kiva.fonttools.font import Font
import numpy as np
import random
from numpy import arange, array, hstack, random
from matplotlib.pyplot import hist
from scipy.optimize import leastsq
from psd import psd
#}}}

#{{{ Time Series

class TimeSeries(HasTraits):
    '''
    A class to store the acquired time series data
    and plot it
    '''

#{{{ Traits definitions

    #Traits for holding data
    t = Array()
    data = Array()
    cidx = CInt(0) #Current index

    #Data properties
    fsamp = CFloat(100.0)    
    selectedChans = List([0])
    ChNames = List(['CH1'])
    nCh = CInt(1) #Number of channels
    nSamples = CInt(1)
    isRunning = Bool(False)
    chx = Enum(values='selectedChans') #1st channel for XY plot
    chy = Enum(values='selectedChans') #2st channel for XY plot    

    #Statistical info
    mean0 = CFloat(0.0)
    std0 = CFloat(0.0)
    var0 = CFloat(0.0)

    mean1 = CFloat(0.0)
    std1 = CFloat(0.0)
    var1 = CFloat(0.0)

    mean2 = CFloat(0.0)
    std2 = CFloat(0.0)
    var2 = CFloat(0.0)
    
    mean3 = CFloat(0.0)
    std3 = CFloat(0.0)
    var3 = CFloat(0.0)


    #Plot panel
    plot = Instance(Plot)

    #Plot Properties
    timeRange = CFloat(1.0) #Max time range while updating
    tMin = CFloat(0) #Lower limit of t of the plot
    tMax = CFloat(1) #Upper limit of t of the plot
    plotType = Enum('Time Series', 'X-Y')
    colorList = List(['blue', 'red', 'green', 'black'])
    marker = marker_trait('circle')
    isXYPlot = Bool(False)

    #Full Range
    fullRangeBtn = Button()
    
    #Histogram window
    histoWinList = List([])
    histoBtn = Button()


    
    #Spectrum window
    speWinList = List([])
    speBtn = Button()

#}}}

#{{{ View definition

    #Default View
    traits_view = View(
        HGroup(Item('plot', editor=ComponentEditor(), show_label=False,
                            width=-650, height=-550,resizable=False, padding=10),
               VGroup(Readonly('nCh', label='Number of Channels'),
                      HGroup(Item('timeRange', label='Show last '),Label('second')),
                      HGroup(Item('tMin',label='Show from'),Item('tMax',label='to'),Label('second'),
                             enabled_when='not isRunning'),
                      HGroup(spring,
                             Item('fullRangeBtn', label='Full Range', enabled_when='not isRunning',
                                  show_label=False),spring),
                      Item('plotType', label='Plot Type', style='custom', enabled_when='nCh > 1'),
                      HGroup(spring, Item('chx',label='X Channel'),spring, Item('chy', label='Y Channel'),
                             spring, enabled_when='isXYPlot'),
                      Item('marker',label='Marker', enabled_when='isXYPlot'),
                      VGroup(
                      HGroup(VGroup(spring,Item(label='CH0:'),spring),
                             VGroup(Item('mean0',label='Mean'), Item('std0',label='Standard Deviation'),
                             Item('var0',label='Variance')), style='readonly',
                             visible_when='selectedChans.count(0) == 1'),
                      HGroup(VGroup(spring,Item(label='CH1:'),spring),
                             VGroup(Item('mean1',label='Mean'), Item('std1',label='Standard Deviation'),
                             Item('var1',label='Variance')), style='readonly',
                             visible_when='selectedChans.count(1) == 1'),
                      HGroup(VGroup(spring,Item(label='CH2:'),spring),
                             VGroup(Item('mean2',label='Mean'), Item('std2',label='Standard Deviation'),
                             Item('var2',label='Variance')), style='readonly',
                             visible_when='selectedChans.count(2) == 1'),
                      HGroup(VGroup(spring,Item(label='CH3:'),spring),
                             VGroup(Item('mean3',label='Mean'), Item('std3',label='Standard Deviation'),
                             Item('var3',label='Variance')), style='readonly',
                             visible_when='selectedChans.count(3) == 1'), label='Statistics'),
                      HGroup(spring, Item('histoBtn', label='Histogram', show_label=False,
                                          enabled_when='not isRunning'), spring,
                             Item('speBtn', label='Spectrum', show_label=False,
                                          enabled_when='not isRunning'), spring)
                      )
               ),
        width=1000,
        resizable=True)


#}}}

#{{{ __init__

    def __init__(self):
        super(TimeSeries, self).__init__()
        x = np.array([0])
        y = np.array([0])
        plotdata = ArrayPlotData(t = x, data1 = y)
        plot = Plot(plotdata)
        self.renderer = []
        self.renderer + plot.plot(("t", "data1"), name='ch1', type="line", color="blue")
        self.plot = plot

        self.plot.index_axis.title = 'Time [sec]'
        self.plot.index_axis.title_font = Font(size=22)
        self.plot.index_axis.tick_label_font = Font(size=16)
        self.plot.value_axis.title = 'Voltage'
        self.plot.value_axis.title_font = Font(size=22)
        self.plot.value_axis.tick_label_font = Font(size=16)
        self.legend = Legend(component=self.plot, align="ur")
        self.plot.overlays.append(self.legend)

        self.plot.tools.append(PanTool(self.plot, drag_button='right'))
        self.plot.tools.append(ZoomTool(self.plot))        
        self.plot.tools.append(RectZoomTool(self.plot, color='dimgrey'))

        #self.plot.index_scale = 'log'
        #self.plot.value_scale = 'log'

#}}}

#{{{ def prepare(self, nSamples, selectedChans, fsamp):

    def prepare(self, nSamples, selectedChans, fsamp):
        '''
        Prepare TimeSeries for new data
        '''
        self.nSamples = nSamples
        self.selectedChans = selectedChans
        self.nCh = len(selectedChans)
        self.fsamp = fsamp

        #Switch to Time Series Displya
        self.plotType = 'Time Series'

        
        #Clear current plots
        plotNames = self.plot.plots.keys()
        [self.plot.delplot(name) for name in plotNames]
        self.renderer = []

        
        self.t = np.arange(self.nSamples)/self.fsamp
        self.data = np.empty((self.nCh, self.nSamples))
        self.cidx = 0

        #Add plots
        self.plot.data.set_data('t',[0])
        for ii in range(self.nCh):
            chidx = str(selectedChans[ii])
            self.plot.data.set_data('data'+chidx, [0]) 
            self.renderer = self.renderer + self.plot.plot(('t', 'data'+chidx),
                                           name='ch'+chidx,
                                           type='line',
                                           color=self.colorList[int(chidx)])

        #Set auto range
        self.plot.value_range.high_setting = 'auto'
        self.plot.value_range.low_setting = 'auto'        
        self.plot.index_range.high_setting = 'auto'
        self.plot.index_range.low_setting = 'auto'        
            
        #Add Legends        
        self.legend.plots = self.plot.plots
        self.legend.visible = True


        #Adjust plot position
        self.plot.outer_position=(50,10)


        #Rest the statistics
        self.clear_statistics()

        #Initialize XY Plot Channels
        self.chx = self.selectedChans[0]
        self.chy = self.selectedChans[-1]
        #self.set(trait_change_notify=False, chx = self.selectedChans[0])
        #self.set(trait_change_notify=False, chy = self.selectedChans[-1])

#}}}

#{{{ def add_data(self, new_data):

    def add_data(self, new_data):
        if np.shape(new_data)[0] != self.nCh:
            print('Error: Data dimension mismatch !')
            return

        nSamplesRetrieved = np.shape(new_data)[1]
        self.data[:,self.cidx:self.cidx+nSamplesRetrieved] = new_data
        self.cidx = self.cidx + nSamplesRetrieved

#}}}

#{{{ def redraw(self):

    def redraw(self):
        '''
        Update the plot when data is changed
        '''
        nSamplesToShow = int(self.fsamp * self.timeRange)

        #Construnct indices
        if (nSamplesToShow > self.cidx) or self.isXYPlot:
            lidx = 0
        else:
            lidx = self.cidx-nSamplesToShow

        #Update t
        self.plot.data.set_data('t',self.t[lidx:self.cidx])
        
        #Update data
        for ii in range(self.nCh):
            chid = str(self.selectedChans[ii])
            self.plot.data.set_data('data'+chid, self.data[ii, lidx:self.cidx])

#}}}

#{{{ def update_statistics(self):

    def update_statistics(self):
        for ii in range(self.nCh):
            chidx = self.selectedChans[ii]
            setattr(self, 'mean'+str(chidx), self.data[ii,:].mean())
            setattr(self, 'std'+str(chidx), self.data[ii,:].std())
            setattr(self, 'var'+str(chidx), self.data[ii,:].var())

#}}}

#{{{ def clear_statistics(self):


    def clear_statistics(self):
        for ii in range(4):
            setattr(self, 'mean'+str(ii), 0)
            setattr(self, 'std'+str(ii), 0)
            setattr(self, 'var'+str(ii), 0)
#}}}

#{{{ def save_ascii(self):

    def save_ascii(self):
        '''
        Save data to file
        '''
        file_name = file_dialog.save_file(file_name='/home/aso', filter=['*.txt', '*.dat'])
        if file_name != '':
            f = open(file_name, 'w')
            f.write('#t')
            for ii in range(self.nCh):
                chid = str(self.selectedChans[ii])
                f.write(','+'Ch'+chid)
            f.write('\n')

            np.savetxt(f, np.vstack((self.t, self.data)).T, delimiter=',', fmt='%1.6e')

            f.close()

#}}}

#{{{ def open_ascii(self):

    def open_ascii(self):
        '''
        Open data to file
        '''
        file_name = file_dialog.open_file(file_name='/home/aso', filter=['*.txt', '*.dat'])
        if file_name != '':
            #Open file
            f = open(file_name, 'r')
            #Read headder
            head = f.readline()
            #Read data
            data = np.loadtxt(f, delimiter=',', unpack=True)
            #Close file
            f.close()

            #Parameters
            nSamples = len(data[0])
            fsamp = 1.0/(data[0,1]-data[0,0])
            chList = head[1:-1].split(',')[1:]
            selectedChans = [int(x[2:]) for x in chList]

            #Prepare the time series
            self.prepare(nSamples=nSamples, selectedChans=selectedChans, fsamp=fsamp)
            #Inject data
            self.add_data(data[1:])
            
            #Show the full range
            self.tMin = 0
            self.tMax = data[0,-1]
            self.update_statistics()

            # for ch in chList:
            #     setattr(self.app.window.controller, ch, True)

            # self.app.window.controller.fsamp = fsamp
            # self.app.window.controller.acqLength = nSamples/fsamp

                         
#}}}

#{{{ Notification handlers

    def _cidx_changed(self, old, new):
        '''
        Update the plot when data is changed
        '''
        self.redraw()

    def _timeRange_changed(self, old, new):
        self.redraw()

    def _plotType_changed(self, old, new):
        if self.plotType == 'X-Y' and self.nCh < 2:
            print('Error: Selected number of channels must be >= 2')
            return

        #Clear current plots
        plotNames = self.plot.plots.keys()
        [self.plot.delplot(name) for name in plotNames]
        self.renderer = []

        if self.plotType == 'X-Y':
            self.isXYPlot = True
            #Add plots
            chid1 = str(self.chx)
            chid2 = str(self.chy)
            self.renderer = self.renderer + self.plot.plot(('data'+chid1, 'data'+chid2),
                                        name='xy',
                                        type='scatter',
                                        color=self.colorList[0])
            self.renderer[0].marker = self.marker
            self.plot.outer_bounds=(550, 550)
            self.plot.index_axis.title = 'Ch'+chid1+' [V]'
            self.plot.value_axis.title = 'Ch'+chid2+' [V]'
            self.legend.visible = False
            self.plot.request_redraw()

        else:
            self.isXYPlot = False

            #Update data
            for ii in range(self.nCh):
                chid = str(self.selectedChans[ii])
                self.renderer = self.renderer + self.plot.plot(('t', 'data'+chid),
                                           name='ch'+chid,
                                           type='line',
                                           color=self.colorList[int(chid)])
            self.plot.outer_bounds=(550, 550)
            self.plot.index_axis.title = 'Time [sec]'
            self.plot.value_axis.title = 'Voltage'
            self.legend.plots = self.plot.plots
            self.legend.visible = True
            self.plot.request_redraw()

        if not self.isRunning:
            self._fullRangeBtn_fired()


    def _marker_changed(self):
        for r in self.renderer:
            r.marker = self.marker

    def _tMin_changed(self, old, new):
        '''
        Update the plot when tMin or tMax is changed
        '''
        
        #Ensure the validity of tMin and tMax
        if self.tMin > self.tMax:
            self.tMin = self.tMax

        if self.tMin < 0:
            self.tMin = 0.0


        #Construnct indices
        lidx = np.searchsorted(self.t, self.tMin)
        uidx = np.searchsorted(self.t, self.tMax)


        #Update t
        self.plot.data.set_data('t',self.t[lidx:uidx])

        #Update data
        for ii in range(self.nCh):
            chid = str(self.selectedChans[ii])
            self.plot.data.set_data('data'+chid, self.data[ii, lidx:uidx])

        self.plot.request_redraw()
        
    def _tMax_changed(self, old, new):

        if self.tMax < 0:
            self.tMax = 1.0

        if self.tMax > self.t[self.cidx-1]:
            self.tMax = self.t[self.cidx-1]

        self._tMin_changed(0,0)

    def _fullRangeBtn_fired(self):
        self.tMin = 0
        self.tMax = self.t[-1]
        
        if self.isXYPlot:
            lim = np.ceil(max(np.max(self.data), np.abs(np.min(self.data))))
            self.plot.value_range.high = lim
            self.plot.value_range.low = -lim
            self.plot.index_range.high = lim
            self.plot.index_range.low = -lim
        else:
            self.plot.index_range.high = self.t[-1]
            self.plot.index_range.low = 0
            self.plot.value_range.high = np.ceil(np.max(self.data))
            self.plot.value_range.low = np.floor(np.min(self.data))
            

    def _chx_changed(self, new, old):

        if self.isXYPlot:
            #Clear current plots
            plotNames = self.plot.plots.keys()
            [self.plot.delplot(name) for name in plotNames]
            self.renderer = []

            self.isXYPlot = True
            #Add plots
            chid1 = str(self.chx)
            chid2 = str(self.chy)
            self.renderer = self.renderer + self.plot.plot(('data'+chid1, 'data'+chid2),
                                        name='xy',
                                        type='scatter',
                                        color=self.colorList[0])
            self.renderer[0].marker = self.marker
            self.plot.outer_bounds=(550, 550)
            self.plot.index_axis.title = 'Ch'+chid1+' [V]'
            self.plot.value_axis.title = 'Ch'+chid2+' [V]'
            self.legend.visible = False
            self.plot.request_redraw()

            if not self.isRunning:
                self._fullRangeBtn_fired()

    def _chy_changed(self, new, old):
        self._chx_changed(1,1)

    def _histoBtn_fired(self):
        #Create a new histogram window
        histoWin = Histogram(self)
        self.histoWinList.append(histoWin)        
        histoWin.edit_traits()

    def _speBtn_fired(self):
        #Create a new histogram window
        speWin = Spectrum(self)
        self.speWinList.append(speWin) 
        speWin.edit_traits()


#}}}            

    
#}}}

#{{{ Histogram

class Histogram(HasTraits):
    '''
    A window to show histograms.
    '''

#{{{ Traits definitions

    #TimeSeries object
    TS = Instance(TimeSeries)

    #Plot panel
    container = Instance(Component)

    #Compute button
    compute_btn = Button()

    #Update button
    updateBtn = Button()

    #Fitting information area
    fitParams = Str()
    
    #Histogram Param
    fitType = Enum('Gaussian', 'Boltzmann')
    fitFormula = Str('y = A*exp[-(x-x0)^2/(2*s^2)]')
    chList = List([0])
    ch = Enum(values='chList')  #channel to be plotted when fitType is Gaussian
    nbin = CInt(20) #Number of bins
    ch1 = Enum(values='chList')  #1st channel when fitType is Boltzmann
    ch2 = Enum(values='chList')  #2nd channel when fitType is Boltzmann
    normed = Enum(False, True)  #Whether the histograms are normalized or not

    weight = Enum(False, True)
    promode = Bool(False)

#}}}

#{{{ View definition

    #Default View
    traits_view = View(
        VGroup(Item('container', editor=ComponentEditor(), show_label=False,
                            width=-650, height=-550,resizable=False, padding=10),
               HGroup(Item('fitType', label='Fitting Type', style='custom'),
                      Item('nbin',label='Number of bins'),
                      Item('ch', label='Channel', visible_when='fitType == "Gaussian"'),
                      Item('ch1', label='1st Channel', visible_when='fitType == "Boltzmann"'),
                      Item('ch2', label='2nd Channel', visible_when='fitType == "Boltzmann"'),
                      Item('normed',label='Normalized', style='custom')
                      #Item('weight', label='Fit with weight', visible_when='promode')
                      ),
               Item('fitFormula',label='Fitting Function', style='readonly'),
               Item('fitParams', label='Fitting Results', style='readonly'),
               HGroup(spring,Item('compute_btn', label='Compute', show_label=False),
                      spring,
                      Item('updateBtn', label='Update', show_label=False),spring)
               ),
        resizable=True,
        kind='live',
        title='Histogram')

#}}}

#{{{ __init__()

    def __init__(self, ts):
        self.TS = ts
        self.chList = self.TS.selectedChans
        self.container = OverlayPlotContainer(bgcolor = 'white')
        self.data_all = self.TS.data
        self._draw_single_hist()

#}}}

#{{{ def _draw_single_hist(self):

    def _draw_single_hist(self):
        chIdx = self.chList.index(self.ch)
        data = self.data_all[chIdx]
        (n, bin) = np.histogram(data, bins = self.nbin, normed=self.normed)
        x = (bin[:-1]+bin[1:])/2.0
        binwidth = bin[1]-bin[0]

        idx = ArrayDataSource(x)
        vals = ArrayDataSource(n)
        
        index_range = DataRange1D(idx, low=np.min(bin), high=np.max(bin))
        index_mapper = LinearMapper(range=index_range)

        value_range = DataRange1D(low=0, high=np.max(n))
        value_mapper = LinearMapper(range=value_range)

        if self.normed:
            plot = ScatterPlot(index=idx, value=vals,
                               value_mapper=value_mapper,
                               index_mapper=index_mapper,
                               color='blue',
                               marker='circle',
                               marker_size=10.0,
                               antialias=True)
        else:
            plot = BarPlot(index=idx, value=vals,
                           value_mapper=value_mapper,
                           index_mapper=index_mapper,
                           line_color='black',
                           fill_color='blue',
                           bar_width=binwidth, antialias=True)

        plot.padding = 20
        plot.padding_left = 80 
        plot.padding_bottom = 60
        plot.border_visible = True
        self.container.add(plot)

        #Add axes
        if self.normed:
            ytitle = 'Probability Density: P(x)'
        else:
            ytitle = 'Number'

        left_axis = PlotAxis(plot, orientation='left', tick_label_font=Font(size=20),
                             title=ytitle, title_font=Font(size=20))
        bottom_axis = PlotAxis(plot, orientation='bottom', tick_label_font=Font(size=20),
                               title='Voltage [V]', title_font=Font(size=20))
        plot.underlays.append(left_axis)
        plot.underlays.append(bottom_axis)

        #Add grids
        hgrid = PlotGrid(component=plot, orientation='horizontal', line_color='gray',
                         line_style='dash', mapper=value_mapper)
        vgrid = PlotGrid(component=plot, orientation='vertical', line_color='gray',
                         line_style='dash', mapper=value_mapper)
        plot.underlays.append(hgrid)
        plot.underlays.append(vgrid)        

        #Gaussian Fit
        param = self._gaussian_fit(x,n)

        xfit = np.linspace(np.min(x), np.max(x), 1e3)
        nfit = param[0]*np.exp(-(xfit-param[1])**2/(2*param[2]**2))

        idx = ArrayDataSource(xfit)
        vals = ArrayDataSource(nfit)

        index_range = DataRange1D(idx, low=np.min(bin), high=np.max(bin))
        index_mapper = LinearMapper(range=index_range)

        value_range = DataRange1D(low=0, high=np.max(nfit))
        value_mapper = LinearMapper(range=value_range)
        
        plotfit = LinePlot(index=idx, value=vals,
                       value_mapper=value_mapper,
                       index_mapper=index_mapper,
                       line_color='black')
        plotfit.padding = 20
        plotfit.padding_left = 80 
        plotfit.padding_bottom = 60
        plotfit.border_visible = True
        self.container.add(plotfit)

#}}}        

#{{{ def _gaussian_fit(self, x, n):

    def _gaussian_fit(self, x, n):
        chIdx = self.chList.index(self.ch)
        data = self.data_all[chIdx]

        #Mean and STD
        m = np.mean(data)
        s = np.std(data)

        #Fitting function
        fitFunc = lambda p,x1: p[0]*np.exp(-(x1-p[1])**2/(2*p[2]**2))

        #Error function
        errFunc = lambda p,x1,y1: fitFunc(p, x1) - y1

        #Initial value
        p0=[np.max(n), m, s]

        #Fitting
        (p, cov_x, infodict, mesg, ierr) = leastsq(errFunc, p0[:], args=(x,n), full_output=True)

        res = np.sum(infodict['fvec']**2)
        
        self.fitParams = \
                       'A = '+str(p[0]) + \
                       ',  x0 = '+str(p[1]) +\
                       ',  s = '+str(p[2]) +\
                       ',  residual = '+str(res)

        print('Fitting message: '+mesg)
        return p

#}}}

#{{{ def _draw_boltzmann(self):

    def _draw_boltzmann(self):
        chIdx1 = self.chList.index(self.ch1)
        chIdx2 = self.chList.index(self.ch2)
        data = self.data_all[chIdx1]**2 + self.data_all[chIdx2]**2

        (n, bin) = np.histogram(data, bins = self.nbin, normed=self.normed)
        x0 = (bin[:-1]+bin[1:])/2.0
        binwidth = bin[1] - bin[0]

        #Remove singular points
        if self.normed:
            nonzero_idx = np.nonzero(n != 0)
            nlog = np.log(n[nonzero_idx])
            x = x0[nonzero_idx]
        else:
            zero_idx = np.nonzero(n == 0)
            n[zero_idx] = 1
            nlog = np.log(n)
            x = x0
        
        idx = ArrayDataSource(x)
        vals = ArrayDataSource(nlog)
        
        index_range = DataRange1D(idx, low=np.min(bin), high=np.max(bin))
        index_mapper = LinearMapper(range=index_range)

        if self.normed:
            value_range = DataRange1D(low=np.min(nlog), high=0)
        else:
            value_range = DataRange1D(low=0, high=np.max(nlog))
        value_mapper = LinearMapper(range=value_range)

        if self.normed:
            plot = ScatterPlot(index=idx, value=vals,
                               value_mapper=value_mapper,
                               index_mapper=index_mapper,
                               color='blue',
                               marker='circle',
                               marker_size=10.0,
                               antialias=True)
        else:
            plot = BarPlot(index=idx, value=vals,
                           value_mapper=value_mapper,
                           index_mapper=index_mapper,
                           line_color='black',
                           fill_color='blue',
                           bar_width=binwidth, antialias=True)

        plot.padding = 20
        plot.padding_left = 80 
        plot.padding_bottom = 60
        plot.border_visible = True
        self.container.add(plot)

        #Add axes
        if self.normed:
            ytitle = 'Logarithmic Probability Density: log(P(x))'
        else:
            ytitle = 'log(Number)'
            
        left_axis = PlotAxis(plot, orientation='left', tick_label_font=Font(size=20),
                             title=ytitle, title_font=Font(size=20))
        bottom_axis = PlotAxis(plot, orientation='bottom', tick_label_font=Font(size=20),
                               title='Power [V^2]', title_font=Font(size=20))
        plot.underlays.append(left_axis)
        plot.underlays.append(bottom_axis)

        #Add grids
        hgrid = PlotGrid(component=plot, orientation='horizontal', line_color='gray',
                         line_style='dash', mapper=value_mapper)
        vgrid = PlotGrid(component=plot, orientation='vertical', line_color='gray',
                         line_style='dash', mapper=value_mapper)
        plot.underlays.append(hgrid)
        plot.underlays.append(vgrid)

        #Perform Fitting
        param = self._boltzmann_fit(x0,n)

        #Plot Fit Result
        xfit = np.linspace(np.min(x), np.max(x), 1e3)
        nfit = param[0]*xfit + param[1]

        idx = ArrayDataSource(xfit)
        vals = ArrayDataSource(nfit)

        index_range = DataRange1D(idx, low=np.min(bin), high=np.max(bin))
        index_mapper = LinearMapper(range=index_range)

        if self.normed:
            value_range = DataRange1D(low=np.min(nfit), high=0)
        else:
            value_range = DataRange1D(low=0, high=np.max(nfit))

        value_mapper = LinearMapper(range=value_range)
        
        plotfit = LinePlot(index=idx, value=vals,
                       value_mapper=value_mapper,
                       index_mapper=index_mapper,
                       line_color='black')
        plotfit.padding = 20
        plotfit.padding_left = 80 
        plotfit.padding_bottom = 60
        plotfit.border_visible = True
        self.container.add(plotfit)

#}}}

#{{{ def _boltzmann_fit(self, x, n):

    def _boltzmann_fit(self, x, n):

        #Remove singular points
        nonzero_idx = np.nonzero(n != 0)
        nlog = np.log(n[nonzero_idx])
        x = x[nonzero_idx]
        
        #Initial guess
        a = (nlog[-1] - nlog[0])/(x[-1] - x[0])
        b = nlog[0]
        
        #Fitting function
        #fitFunc = lambda x1, p0, p1: p0*x1 + p1

        #Error function
        if self.weight:
            errFunc = lambda p, x1, y1: (y1 - (p[0]*x1 + p[1]))/y1
        else:
            errFunc = lambda p, x1, y1: y1 - (p[0]*x1 + p[1]) 

        #Initial value
        p0=[a,b]

        #Fitting
        (p, cov_x, infodict, mesg, ierr) = leastsq(errFunc, p0, args=(x,nlog), full_output=True)
        
        res = np.sum(infodict['fvec']**2)
        
        self.fitParams = \
                       'a = '+str(p[0]) + \
                       ',  b = '+str(p[1]) +\
                       ',  residual = '+str(res)

        print('Fitting message: '+mesg)

        return p

#}}}

#{{{ Notification Handlers

    def _compute_btn_fired(self):

        #Clear old plots
        while len(self.container.components) > 0:
            for c in self.container.components:
                self.container.remove(c)

        if self.fitType == 'Boltzmann':
            self._draw_boltzmann()
        else:
            self._draw_single_hist()

        self.container.request_redraw()

    def _updateBtn_fired(self):
        self.chList = self.TS.selectedChans
        self.data_all = self.TS.data
        self._compute_btn_fired()

    def _fitType_changed(self, old, new):
        if self.fitType == 'Gaussian':
            self.fitFormula = 'y = A*exp[-(x-x0)^2/(2*s^2)]'
        else:
            self.fitFormula = 'y = a*x + b'
            self.ch2 = self.chList[-1]

        self._compute_btn_fired()

    def _ch_changed(self, old, new):
        self._compute_btn_fired()

    def _ch1_changed(self, old, new):
        self._compute_btn_fired()

    def _ch2_changed(self, old, new):
        self._compute_btn_fired()

    def _normed_changed(self, old, new):
        self._compute_btn_fired()

#}}}

#}}}

#{{{ Spectrum 

class Spectrum(HasTraits):
    '''
    A class to hold frequency spectrum data
    '''

#{{{ Traits definitions

    #Traits for holding data
    f = Array()
    data = Array()

    #TimeSeries object
    TS = Instance(TimeSeries)

    #Plot panel
    plot = Instance(Plot)

    #Compute button
    compute_btn = Button()

    #Update button
    updateBtn = Button()

    #PSD Param
    chList = List([0])
    ch = Enum(values='chList')  #channel to be plotted
    navg = CInt(5) #Number of average
    overlap = Enum('0%','25%','50%','75%')
    window = Enum('None', 'Hanning', 'Hamming')

#}}}

#{{{ View definition

    #Default View
    traits_view = View(
        VGroup(Item('plot', editor=ComponentEditor(), show_label=False,
                            width=-650, height=-550,resizable=False, padding=10),
               HGroup(Item('ch', label='Channel'),
                      Item('navg',label='Number of Averages'),
                      Item('overlap',label='Overlap'),
                      Item('window',label='Window')),
               HGroup(spring,Item('compute_btn', label='Compute', show_label=False),
                      spring,
                      Item('updateBtn', label='Update', show_label=False),spring)
               ),
        resizable=True,
        width=800,
        kind='live',
        title='Spectrum')

#}}}

#{{{ __init__()
    def __init__(self, ts):
        self.TS = ts
        self.chList = self.TS.selectedChans
        self.fs = self.TS.fsamp
        self.data_all = self.TS.data

        x = np.array([0])
        y = np.array([0])
        plotdata = ArrayPlotData(t = x, data1 = y)
        self.plot = Plot(plotdata)
        self.renderer = []

        self.plot.index_scale = 'log'
        self.plot.value_scale = 'log'

        self.plot.index_axis.title = 'Frequency [Hz]'
        self.plot.index_axis.title_font = Font(size=22)
        self.plot.index_axis.tick_label_font = Font(size=16)
        self.plot.value_axis.title = 'PSD [V/sqrt(Hz)]'
        self.plot.value_axis.title_font = Font(size=22)
        self.plot.value_axis.tick_label_font = Font(size=16)

        self.plot.tools.append(PanTool(self.plot, drag_button='right'))
        self.plot.tools.append(ZoomTool(self.plot))        
        self.plot.tools.append(RectZoomTool(self.plot, color='dimgrey'))

        #Select a channel and compute a PSD
        self.ch = self.chList[0]
        self.overlap = '25%'
        self.window = 'Hanning'
        self.compute_psd()

#}}}

#{{{ def scientific_formatter(self,x):
    def scientific_formatter(self,x):
        if x < 1e-2 or x > 1e3:
            return '%.1e'%x
        else:
            return '%.1f'%x        
#}}}

#{{{ compute_psd

    def compute_psd(self):

        #Get Data
        chIdx = self.chList.index(self.ch)
        data = self.data_all[chIdx]

        #Sampling Frequency
        fs = self.fs

        overlap = {'0%': 0.0, '25%': 0.25,
                   '50%': 0.5, '75%': 0.75}[self.overlap]

        window = {'None': None, 'Hanning': np.hanning,
                  'Hamming': np.hamming}[self.window]

        #Compute PSD
        (f, p) = psd(data, fs=fs, navg=self.navg, overlap=overlap,
                     window=window)

        #Remove DC
        f = f[1:]
        p = p[1:]

        #Plot PSD

        #Clear current plots
        plotNames = self.plot.plots.keys()
        [self.plot.delplot(name) for name in plotNames]
        self.renderer = []

        self.plot.value_axis.tick_label_formatter = self.scientific_formatter
        
        #Add plot
        self.plot.data.set_data('f',f)
        self.plot.data.set_data('psd',p)
        self.renderer = self.renderer + self.plot.plot(('f', 'psd'),
                                                       name='psd',
                                                       type='line',
                                                       color='blue')

        self.plot.index_scale = 'log'
        self.plot.value_scale = 'log'
        #Adjust plot position
        #self.plot.outer_position=(50,10)
        self.plot.padding_bottom=60
        self.plot.padding_left=100

        self.plot.request_redraw()

#}}}

#{{{ Notification Handlers

    def _compute_btn_fired(self):
        self.compute_psd()

    def _updateBtn_fired(self):
        self.chList = self.TS.selectedChans
        self.fs = self.TS.fsamp
        self.data_all = self.TS.data
        self._compute_btn_fired()
        

#}}}

#}}}
















































































