#{{{ Import modules
from enthought.traits.api import HasTraits, Instance, Int, Float, Enum, Bool,\
     Trait, Callable, Button, Array, CInt, CFloat, List
import numpy as np
import numpy.random as random
#}}}


#{{{ DAQ Core
class DAQCore(HasTraits):
    '''
    A class to represent a DAQ hardware.
    This is meant to be the parent class for actual DAQCore classes.
    '''

    #DAQ parameters
    fsamp = CFloat(1000.0) #Sampling rate [Hz]
    nCh = CInt(1)  #Number of channels
    acqLength = CFloat(10.0)  #Acquisition length [sec]
    nSamples = CInt(0)  #Number of samples to acquire

    def __init__(self, fsamp=100.0,  Vrange=10.0, selectedChans=[0], acqLength=10.0):
        self.fsamp = fsamp
        self.selectedChans = selectedChans
        self.nCh = len(selectedChans)
        self.Vrange = Vrange
        self.acqLength = acqLength
        self.nSamples = int(np.ceil(self.fsamp*self.acqLength))

    def start(self, fsamp=100.0, Vrange=10.0, selectedChans=[0], acqLength=10.0):
        '''
        Start the DAQ process. Overwrite this function with a real
        function to start a real DAQ hardware.
        '''
        self.fsamp = fsamp
        self.selectedChans = selectedChans
        self.nCh = len(selectedChans)
        self.Vrange = Vrange
        self.acqLength = acqLength
        self.nSamples = int(np.ceil(self.fsamp*self.acqLength))

        print('DAQ Started')

    def stop(self):
        print('DAQ Stopped')

    def readdata(self, n, data):
        '''
        Read n samples from the DAQ buffer.
        data must be an ndarray of size (n, nCh).
        It has to be created with numpy.ascontiguousarray() so that the memory
        access is linear.
        '''
        return np.zeros((self.nCh, n))

        
#}}}

#{{{ Fake DAQ Core
class FakeDAQCore(DAQCore):
    #Fake Data Generator Parameters
    mean = 0.0
    stddev = 1.0
    fakedata = Array()
    idx = Int(0)

    def __init__(self, fsamp=100.0, Vrange=10.0, selectedChans=[0], acqLength=10.0):
        super(FakeDAQCore, self).__init__(fsamp, Vrange, selectedChans, acqLength)

    def start(self, fsamp=100.0, Vrange=10.0, selectedChans=[0], acqLength=10.0):
        super(FakeDAQCore, self).__init__(fsamp, Vrange, selectedChans, acqLength)

        #Generate fake data at once
        self.fakedata = random.normal(self.mean, self.stddev, (self.nSamples, self.nCh))
        self.idx = 0
        
        print('DAQ Started')

    def stop(self):
        print('DAQ Stopped')

    def readdata(self, n, data):
        '''
        Read n samples from the DAQ buffer
        '''
        lidx = self.idx
        uidx = lidx+n
        self.idx = self.idx + n
        if uidx > self.nSamples:
            uidx = -1
        if lidx > self.nSamples:
            lidx = -1
            
        data[:n,:] = self.fakedata[lidx:uidx,:]
        return n

#}}}
