#{{{ Import modules
from enthought.traits.api import HasTraits, Instance, Int, CInt, Float, CFloat,\
     Enum, Bool, Trait, Callable, Button, Array, List
from enthought.traits.ui.api import View, Group, HGroup, Item, Readonly, spring
from enthought.traits.ui.menu import Action, MenuBar, Menu, CloseAction
from enthought.enable.api import ColorTrait
from enthought.enable.component_editor import ComponentEditor
from enthought.chaco.api import marker_trait, Plot, ArrayPlotData
import numpy as np
import random
from numpy import arange, array, hstack, random
import wx
#}}}

#{{{ Scatter Plot Class

class PlotPanel(HasTraits):

    plot = Instance(Plot)
    color = ColorTrait("blue")

    traits_view = View(
        Group(Item('color', label="Color", style="custom"),
              Item('plot', editor=ComponentEditor(), show_label=False),
                   orientation = "vertical"),
              width=800, height=600, resizable=True)

    def __init__(self):
        super(PlotPanel, self).__init__()
        t = np.array([0])
        y = np.array([0])
        plotdata = ArrayPlotData(t = t, data1 = y)
        plot = Plot(plotdata)
        self.renderer = plot.plot(("t", "data1"), type="line", color="blue")[0]
        self.plot = plot

    def _color_changed(self):
        self.renderer.color = self.color

#}}}


#{{{ Controller Handler

# class CtrHandller(Controller):
#     def start(self, info):
#         info.ctr.start()

                  
#     def stop(self, info):
#         info.ctr.stop()
        

#}}}

#{{{ DAQ Core

class DAQCore(HasTraits):
    '''
    A class to represent a DAQ hardware
    '''
    fsamp = Float(1000.0) #Sampling rate [Hz]
    nCh = Int(1)  #Number of channels
    acqLength = Float(10.0)  #Acquisition length [sec]
    
    mean = 0.0
    stddev = 1.0

    def __init__(self, fsamp=1000.0, nCh=1, acqLength=10.0):
        self.fsamp = fsamp
        self.nCh = nCh
        self.acqLength = acqLength

    def startDAC(self):
        pass
#}}}


#{{{ Controller Class

class DAQController(HasTraits):
    
    # A reference to the plot viewer object
    viewer = Instance(PlotPanel)
    
    # Some parameters controller the random signal that will be generated
    distribution_type = Enum("normal", "lognormal")
    mean = Float(0.0)
    stddev = Float(1.0)

    fsamp = Float(1000.0)
    updateInterval = Float(100.0) # in msec
    numSamplesPerUpdate = Int(int(100*100.0/1000))

    isRunning = Bool(False)

    start_button = Button()
    stop_button = Button()
    
    # private reference to the random number generator.  this syntax
    # just means that self._generator should be initialized to
    # random.normal, which is a random number function, and in the future
    # it can be set to any callable object.
    #_generator = Trait(random.normal, Callable)
    
    traits_view = View(Group(Item('distribution_type',style='custom'),
        Item('fsamp', label='Sampling Rate'), 
        Item('updateInterval', label='Update Interval [msec]'), 
        #Readonly('stddev'),
        HGroup(spring,
               Item('start_button',label='Start',show_label=False, enabled_when='not isRunning'),
               spring,
               Item('stop_button',label='Stop',show_label=False, enabled_when='isRunning'),
               spring),
        orientation="vertical"))
                       

    def __init__(self, app, viewer):
        self.app = app
        self.viewer = viewer
        self.numSamplesPerUpdate = int(self.fsamp*self.updateInterval/1000.0)

    
    def timer_tick(self, *args):
        """ Callback function that should get called based on a wx timer
        tick.  This will generate a new random datapoint and set it on
        the .data array of our viewer object.
        """
        # Generate a new number and increment the tick count
        new_val = random.normal(self.mean, self.stddev, self.numSamplesPerUpdate)
        
        # grab the existing data, truncate it, and append the new point.
        # This isn't the most efficient thing in the world but it works.
        cur_index = self.viewer.plot.data.arrays['x']
        cur_data = self.viewer.plot.data.arrays['y']
        new_data = hstack((cur_data, new_val))
        new_index = hstack((cur_index, arange(1,self.numSamplesPerUpdate+1)/self.fsamp+cur_index[-1]))

        
        self.viewer.plot.data.set_data('x',new_index)
        self.viewer.plot.data.set_data('y',new_data)        
        return

    def _distribution_type_changed(self):
        # This listens for a change in the type of distribution to use.
        if self.distribution_type == "normal":
            self._generator = random.normal
        else:
            self._generator = random.lognormal

    def _start_button_fired(self):
        self.isRunning = True
        self.app.start_timer(self.updateInterval)
        return

    def _stop_button_fired(self):
        self.isRunning = False
        self.app.stop_timer()
        return

#}}}

#{{{
class MainWindow(HasTraits):
    viewer = Instance(PlotPanel)
    controller = Instance(DAQController)

    def __init__(self, app):
        self.viewer = PlotPanel()
        self.controller = DAQController(app=app, viewer = self.viewer)

    traits_view = View(Item('viewer', show_label=False),\
                       Item('controller',show_label=False),\
                       style="custom", resizable=True,
                       menubar=MenuBar(Menu(CloseAction, name = 'File')))
    
#}}}


#{{{ MyApp

class MyApp(wx.PySimpleApp, HasTraits):
    
    window = Instance(MainWindow)
    def OnInit(self, *args, **kw):
        
        self.window = MainWindow(app=self)
        self.window.edit_traits()

        
        # Pop up the windows for the two objects
        #viewer.edit_traits()
        #controller.edit_traits(handler=CtrHandller(), context={'object':controller, 'ctr':controller})
        #controller.edit_traits()
        
        # Set up the timer and start it up
        self.setup_timer(self.window.controller)
        return True


    def setup_timer(self, controller):
        # Create a new WX timer
        timerId = wx.NewId()
        self.timer = wx.Timer(self, timerId)
        
        # Register a callback with the timer event
        self.Bind(wx.EVT_TIMER, controller.timer_tick, id=timerId)

    def start_timer(self, updateInterval):
        # Start up the timer!  We have to tell it how many milliseconds
        # to wait between timer events.
        
        self.timer.Start(updateInterval, wx.TIMER_CONTINUOUS)
        return

    def stop_timer(self):
        self.timer.Stop()
        return

#}}}


#{{{ 

if __name__ == "__main__":
    app = MyApp()
    app.MainLoop() 
else:
    app = MyApp()
    app.MainLoop() 

#}}}
