#{{{ Imports
import ctypes as ct
import numpy as np
import daqcore
#}}}


#{{{ Headder file like stuff

int32 = ct.c_long
uInt32 = ct.c_ulong
uInt64 = ct.c_ulonglong
float64 = ct.c_double
TaskHandle = uInt32

# the constants
DAQmx_Val_Cfg_Default = int32(-1)
DAQmx_Val_RSE = 10083
DAQmx_Val_NRSE = 10078
DAQmx_Val_Diff = 10106
DAQmx_Val_Volts = 10348
DAQmx_Val_Rising = 10280
DAQmx_Val_FiniteSamps = 10178
DAQmx_Val_GroupByChannel = 0
DAQmx_Val_GroupByScanNumber = 1

#}}}


#{{{ NIDAQ Class

class NIDAQCore(daqcore.DAQCore):
    '''
    A class to represent a DAQ hardware
    '''

    #Load library
    nidaq = ct.CDLL('libnidaqmxbase.so.3', mode=ct.RTLD_GLOBAL)
    
    def __init__(self, fsamp=100.0, Vrange=10.0, selectedChans=[0], acqLength=10.0):
        super(NIDAQCore, self).__init__(fsamp, Vrange, selectedChans, acqLength)

        #Define variables
        self.taskHandle = TaskHandle(0)
        self.time_out = 10.0
        self.read = int32()
        self.nSamplesLeft = self.nSamples

    def start(self, fsamp=100.0, Vrange=10.0, selectedChans=[0], acqLength=10.0):
        super(NIDAQCore, self).__init__(fsamp, Vrange, selectedChans, acqLength)

        #Initialize retrieved sample counters
        self.nSamplesLeft = self.nSamples

        #Prepare a buffer
        #By default, a buffer for 1sec of data is prepared. It is enlarged
        #in readdata() if the user requests longer data.
        self.buff = np.ascontiguousarray(np.empty((int(self.fsamp), self.nCh)))
        
        #Create a task
        self.CHK(self.nidaq.DAQmxBaseCreateTask("",ct.byref(self.taskHandle)))

        #Construct a device/channel string
        devChanStr = ""
        for ii in range(self.nCh):
            devChanStr = devChanStr + "Dev1/ai"+str(self.selectedChans[ii])+", "
        devChanStr = devChanStr[:-2]

        #Create AI Voltage Channels
        self.CHK(self.nidaq.DAQmxBaseCreateAIVoltageChan(self.taskHandle, devChanStr, "",
                                               DAQmx_Val_Diff,
                                               float64(-1*self.Vrange), float64(self.Vrange),
                                               DAQmx_Val_Volts, None))

        #Configure Sampling Frequency and Length
        self.CHK(self.nidaq.DAQmxBaseCfgSampClkTiming(self.taskHandle, "", float64(self.fsamp),
                                            DAQmx_Val_Rising, DAQmx_Val_FiniteSamps,
                                            uInt64(self.nSamples)));

        #Start DAQ process
        self.CHK(self.nidaq.DAQmxBaseStartTask(self.taskHandle))

        print('DAQ Started')

    def stop(self):
        if self.taskHandle.value != 0:
            self.nidaq.DAQmxBaseStopTask(self.taskHandle)
            self.nidaq.DAQmxBaseClearTask(self.taskHandle)

        print('DAQ Stopped')

    def readdata(self, n, data):
        '''
        Read n samples from the DAQ buffer
        data must be an ndarray of size (n, nCh).
        It has to be created with numpy.ascontiguousarray() so that the memory
        access is linear.
        This function returns the actual number of samples read.
        '''

        #Check if the user tries to read too many samples
        if self.nSamplesLeft < n:
            n = self.nSamplesLeft

        if n == 0:
            return 0

        #Check if the size and the type of data is acceptable
        if data.shape != (n, self.nCh) or data.dtype != np.float64:
            print("Error: data is not the right size or type.")
            return -1

        #Prepare a larger buffer if necessary
        if len(self.buff) < n:
            self.buff = np.ascontiguousarray(np.empty((n,self.nCh)))
            
        #Go ahead and read some data
        self.CHK(self.nidaq.DAQmxBaseReadAnalogF64(self.taskHandle, n, float64(self.time_out),
                             DAQmx_Val_GroupByScanNumber, self.buff.ctypes.data,
                             uInt32(data.size), ct.byref(self.read), None))

        #Copy the buffer contents to data
        data[:self.read.value, :] = self.buff[:self.read.value, :]

        #Update the number of samples left
        self.nSamplesLeft = self.nSamplesLeft - self.read.value
        
        return self.read.value

    def CHK(self, err):
        """a simple error checking routine"""
        if err < 0:
            buf_size = 100
            buf = ct.create_string_buffer('\000' * buf_size)
            self.nidaq.DAQmxBaseGetExtendedErrorInfo(err,ct.byref(buf),buf_size)
            raise RuntimeError('nidaq call failed with error %d: %s'%(err,repr(buf.value)))

#}}}
