import numpy as np

def periodgram(x,fs,window=None):
    """
    Compute a one sided periodgram
    """
    N=np.size(x)  #Number of points
    
    if not window is None:
        x=x*window
        windFact=(window**2).sum()/N
    else:
        windFact=1
        


    p=np.fft.fft(x)  #DFFT
    p=1.0/(N*fs)*np.abs(p)**2/windFact  #Periodgram
    p=2*p[:N/2] #Extract the first half and multiply with 2 to conserve the energy.

    #Frequency array
    f=np.fft.fftfreq(N,1.0/fs)
    f=f[:N/2]
    
    return (f,p)

def psd(x,fs,navg=5,overlap=0,window=None):
    """psd(x,fs,navg=5,overlap=0,window=None)
    - Calculates power spectrum density of a given time series.

    x: One dimensional array representing a time series.
    fs: The sampling frequency of the time series.
    navg: The number of averages to be performed. x is divided into
    navg parts and a periodgram is caluculated for each part. Then the periodgrams are averaged to obtain a smooth spectrum.
    overlap: A number between 0 and 1. When overlap is non-zero, neighboring parts are overlapped by this fraction.
    window: window function. numpy.hanning and numpy.hamming are popular ones.
    """
    # Determine the number of samples in each segment
    N=np.floor(np.size(x)/(1.0+(1.0-overlap)*(navg-1)));

    # Make N even
    if np.mod(N,2):
        N=N-1

    # Increment between the first samples of adjacent segments
    Ninc=np.floor(N*(1-overlap));

    #Create window
    if not window is None:
        windata=window(N)
    else:
        windata=None

    #Initialization
    p=np.zeros(N/2)
    f=np.zeros(N/2)
    
    for ii in range(navg):
        # Extract samples to be analyzed
        data=x[ii*Ninc:(ii)*Ninc+N];
        # Calculate periodogram
        (f,p1)=periodgram(data,fs,windata)
        p=p+p1

    p=p/navg 
    return (f,p)
