#!/usr/bin/env python

#{{{ Import modules
import enthought.etsconfig.api
enthought.etsconfig.api.ETSConfig.toolkit = 'wx'
from enthought.traits.api import HasTraits, Instance, Int, Float, Enum, Bool,\
     Trait, Callable, Button, Array, CInt, CFloat, List
from enthought.traits.ui.api import View, Group, HGroup, VGroup, Item, Readonly, spring,\
     Label, Handler
from enthought.traits.ui.menu import Action, MenuBar, Menu, CloseAction
import enthought.traits.ui.file_dialog as file_dialog
from enthought.enable.api import ColorTrait
from enthought.enable.component_editor import ComponentEditor
from enthought.chaco.api import marker_trait, Plot, ArrayPlotData, Legend
from enthought.chaco.tools.api import LegendTool, PanTool, ZoomTool, RectZoomTool
from enthought.kiva.fonttools.font import Font
import numpy as np
import random
from numpy import arange, array, hstack, random
import wx
#import nidaqcore
import daqcore
from analysis import TimeSeries, Histogram
#}}}

#{{{ Controller Class

class DAQController(HasTraits):

#{{{ Traits Definitions

    # DAQ Parameters
    fsamp = Float(1000.0)  #Sampling rate
    VrangeStr = Enum("+/-10V", "+/-5V", "+/-1V", "+/-200mV") #Input Voltage range
    Vrange = Float(10.0)
    nCh = Int(1)
    acqLength = Float(10.0)  #Data acquisition length in sec
    updateInterval = CFloat(100.0)
    nSamplesPerUpdate = Int(1)
    nSamples = CInt(0)  #Number of samples to acquire

    Ch0 = Bool(True)
    Ch1 = Bool(False)
    Ch2 = Bool(False)
    Ch3 = Bool(False)    
    selectedChans = List([1])

    # DAQ Status Variables
    isRunning = Bool(False)
    nSamplesRetrieved = CInt(0)

    start_button = Button()
    stop_button = Button()


#}}}

#{{{ View Definition

    #Default View
    traits_view = View(Group(Item('fsamp', label='Sampling Rate [Hz]', enabled_when='not isRunning'), 
        Item('VrangeStr', label='Input Voltage Range', enabled_when='not isRunning'),
        HGroup(Item('Ch0',label = 'Ch0'),
               Item('Ch1',label = 'Ch1'),
               Item('Ch2',label = 'Ch2'),
               Item('Ch3',label = 'Ch3'), label = 'Input Channels', enabled_when='not isRunning'),
        Item('acqLength', label='Data acquisition length [sec]', enabled_when='not isRunning'),
        HGroup(spring,
               Item('start_button',label='Start',show_label=False, enabled_when='not isRunning'),
               spring,
               Item('stop_button',label='Stop',show_label=False, enabled_when='isRunning'),
               spring),
        orientation="vertical"))
                       
#}}}

#{{{ __init__

    def __init__(self, app):
        self.app = app
        self.nSamples = int(np.ceil(self.fsamp*self.acqLength))
        self.nSamplesPerUpdate = int(self.fsamp*self.updateInterval/1000.0)
        self.Vrange = self.convVrange(self.VrangeStr)

#}}}

#{{{ def _start_button_fired(self):

    def _start_button_fired(self):
        self.isRunning = True

        self.Vrange = self.convVrange(self.VrangeStr)

        #Start DAQ process
        self.selectedChans = list(np.arange(4)[np.array([self.Ch0, self.Ch1, self.Ch2, self.Ch3])])
        self.app.daq.start(fsamp=self.fsamp, Vrange=self.Vrange,
                           selectedChans=self.selectedChans, acqLength=self.acqLength)

        #Prepare buffer for timeseries
        self.app.window.timeseries.prepare(self.nSamples, self.selectedChans, self.fsamp)

        #Set the timeseries to the running mode
        self.app.window.timeseries.isRunning = True
        
        #Prepare local buffer
        self.nCh = len(self.selectedChans)
        self.buff = np.ascontiguousarray(np.empty((self.nSamplesPerUpdate, self.nCh)))

        #Reset retrieved sample counter
        self.nSamplesRetrieved = 0
        
        self.app.start_timer(self.updateInterval)

#}}}        

#{{{ def _stop_button_fired(self):

    def _stop_button_fired(self):

        self.app.daq.stop()
        self.app.stop_timer()
        self.isRunning = False

        #Set the timeseries to not running mode
        self.app.window.timeseries.isRunning = False

        #Check if DAQ is stopped before the end
        if self.nSamplesRetrieved < self.nSamples:
            self.app.window.timeseries.t = self.app.window.timeseries.t[0:self.nSamplesRetrieved]
            self.app.window.timeseries.data = self.app.window.timeseries.data[:,0:self.nSamplesRetrieved]

        #Show the full range
        self.app.window.timeseries.tMin = 0
        self.app.window.timeseries.tMax = self.acqLength
        self.app.window.timeseries.update_statistics()

#}}}

#{{{ def timer_tick(self, *args):

    def timer_tick(self, *args):
        """ Callback function that should get called based on a wx timer
        tick.  
        """
        # Read data from the DAQ hardware
        n = self.app.daq.readdata(self.nSamplesPerUpdate, self.buff)

        if n < 0:
            print('Error: Could not read data')
            return n

        #Update number of samples retrieved so far
        self.nSamplesRetrieved = self.nSamplesRetrieved + n

        #Add the newly retrieved data to timeseries
        self.app.window.timeseries.add_data(self.buff[:n,:].T)

        if self.nSamplesRetrieved >= self.nSamples:
            self._stop_button_fired()

#}}}

#{{{ def convVrange(self, VrangeStr):

    def convVrange(self, VrangeStr):
        if VrangeStr == '+/-10V':
            return 10.0
        elif VrangeStr == '+/-5V':
            return 5.0
        elif VrangeStr == '+/-1V':
            return 1.0
        else:
            return 0.2

#}}}
            
#{{{ Notification functions

    def _fsamp_changed(self, old, new):
        self.nSamples = int(np.ceil(self.fsamp*self.acqLength))
        if self.fsamp > 50:
            self.updateInterval = 100.0
        elif self.fsamp > 10:
            self.updateInterval = 200.0
        else:
            self.updateInterval = 2000.0/self.fsamp
            self.app.window.timeseries.timeRange = 10.0/self.fsamp
            
        self.nSamplesPerUpdate = int(self.fsamp*self.updateInterval/1000.0)

    def _acqLength_changed(self, old, new):
        self.nSamples = int(np.ceil(self.fsamp*self.acqLength))

#}}}        

#}}}

#{{{ Menu Handler

class MainWindowHandler(Handler):
    def save_ascii(self, info):
        #print(dir(info.ui.control))
        #print(info.ui.control.Children)
        #info.ui.control.Close()
        info.object.timeseries.save_ascii()

    def open_ascii(self, info):
        info.object.timeseries.open_ascii()

    def closed(self, info, is_ok):
        super(MainWindowHandler, self).closed(info, is_ok)
        #Exit the application
        info.object.app.Exit()

#}}}

#{{{ Main Window

class MainWindow(HasTraits):
    timeseries = Instance(TimeSeries)
    controller = Instance(DAQController)
    save_ascii = Action(name='Save', action='save_ascii')
    open_ascii = Action(name='Open', action='open_ascii')

    def __init__(self, app):
        self.timeseries = TimeSeries()
        self.controller = DAQController(app=app)
        self.app = app


    traits_view = View(Item('timeseries', show_label=False),\
                       Item('controller',show_label=False),\
                       style="custom", resizable=True,
                       handler=MainWindowHandler(),
                       menubar=MenuBar(Menu(open_ascii, save_ascii, CloseAction,  name = 'File')),
                       title='PyDAQ')


#}}}

#{{{ MyApp

class wxMainApp(wx.PySimpleApp):
    
    def OnInit(self, *args, **kw):
        
        #self.daq = nidaqcore.NIDAQCore()
        self.daq = daqcore.FakeDAQCore()
        self.window = MainWindow(app=self)
        self.window.edit_traits(context={'object':self.window})
        self.setup_timer(self.window.controller)
        return True


    def setup_timer(self, controller):
        # Create a new WX timer
        timerId = wx.NewId()
        self.timer = wx.Timer(self, timerId)
        
        # Register a callback with the timer event
        self.Bind(wx.EVT_TIMER, controller.timer_tick, id=timerId)

    def start_timer(self, updateInterval):
        # Start up the timer!  We have to tell it how many milliseconds
        # to wait between timer events.
        
        self.timer.Start(updateInterval, wx.TIMER_CONTINUOUS)
        return

    def stop_timer(self):
        self.timer.Stop()
        return

#}}}

#{{{ Main  

if __name__ == "__main__":
     app = wxMainApp()
     app.MainLoop() 

#}}}

